@@ -1191,9 +1191,19 @@ let touch ?(create = true) p =
     | In_build_dir k ->
       Kind.to_string (Kind.append_local (Fdecl.get Build.build_dir) k)
   in
-  try Unix.utimes p 0.0 0.0
-  with Unix.Unix_error (Unix.ENOENT, _, _) ->
-    if create then Unix.close (Unix.openfile p [ Unix.O_CREAT ] 0o777)
+  let create =
+    if create then
+      fun () ->
+    Unix.close (Unix.openfile p [ Unix.O_CREAT ] 0o777)
+    else
+      Fun.id
+  in
+  try Unix.utimes p 0.0 0.0 with
+  | Unix.Unix_error (Unix.ENOENT, _, _) -> create ()
+  | Unix.Unix_error (Unix.EUNKNOWNERR 0, _, _)
+    when Sys.win32 && not (Sys.file_exists p) ->
+    (* OCaml PR#8857 *)
+    create ()
 
 let compare x y =
   match (x, y) with
@@ -1259,6 +1269,8 @@ let local_part = function
 
 let stat t = Unix.stat (to_string t)
 
+let lstat t = Unix.lstat (to_string t)
+
 include (Comparator.Operators (T) : Comparator.OPS with type t := t)
 
 let path_of_local = of_local
